print("import: 'rsgislib'")
import rsgislib

print("import: 'rsgislib.imagecalibration'")
import rsgislib.imagecalibration

print("import: 'rsgislib.classification'")
import rsgislib.classification

print("import: 'rsgislib.elevation'")
import rsgislib.elevation

print("import: 'rsgislib.imagecalc'")
import rsgislib.imagecalc

print("import: 'rsgislib.imagefilter'")
import rsgislib.imagefilter

print("import: 'rsgislib.imageregistration'")
import rsgislib.imageregistration

print("import: 'rsgislib.imageutils'")
import rsgislib.imageutils

print("import: 'rsgislib.rastergis'")
import rsgislib.rastergis

print("import: 'rsgislib.segmentation'")
import rsgislib.segmentation

print("import: 'rsgislib.vectorutils'")
import rsgislib.vectorutils

print("import: 'rsgislib.zonalstats'")
import rsgislib.zonalstats

print("import: 'rsgislib.imagemorphology'")
import rsgislib.imagemorphology

