@echo off

IF "%1"=="" (
   echo rsgis-config.bat [OPTIONS]
   echo Options:
   echo     [--prefix]
   echo     [--version]
   echo     [--libs]
   echo     [--cflags]
   echo     [--ldflags]
   echo     [--libdir]
   echo     [--includes]
   EXIT /B 1
) ELSE (
:printValue
    if "%1" neq "" (
	    IF "%1"=="--prefix" echo C:/bld/rsgislib_1565650992549/_h_env/Library
	    IF "%1"=="--version" echo 3.6.14
	    IF "%1"=="--cflags" echo -IC:/bld/rsgislib_1565650992549/_h_env/Library/include
	    IF "%1"=="--libs" echo -LIBPATH:C:/bld/rsgislib_1565650992549/_h_env/Library/lib rsgis_calib.lib  rsgis_classify.lib rsgis_commons.lib rsgis_datastruct.lib rsgis_filter.lib rsgis_geom.lib rsgis_img.lib rsgis_maths.lib rsgis_modeling.lib rsgis_radar.lib rsgis_rastergis.lib rsgis_registration.lib rsgis_segmentation.lib rsgis_utils.lib rsgis_vec.lib
	    IF "%1"=="--ldflags" echo C:/bld/rsgislib_1565650992549/_h_env/Library/lib
	    IF "%1"=="--libdir" echo C:/bld/rsgislib_1565650992549/_h_env/Library/lib
	    IF "%1"=="--includes" echo C:/bld/rsgislib_1565650992549/_h_env/Library/include
		shift
		goto :printValue
    )
	EXIT /B 0
)
